@@ -83,7 +83,9 @@ class TCPConnection(object):
         Failed,
     ))
     _ConnectedStates = set((Connected,))
+    _DisconnectedStates = set((Disconnected, Failed))
     _DoneStates = set((Connected, Disconnected, Failed))
+    _FailedStates = set((Failed,))
     
     def __init__(self,
         host = None,
@@ -175,12 +177,14 @@ class TCPConnection(object):
     def connect(self,
         host=None,
         port=None,
+        timeLim=None,
     ):
         """Open the connection.
 
         Inputs:
         - host: IP address (name or numeric) of host; if omitted, the default is used
         - port: port number; if omitted, the default is used
+        - timeLim: time limit (sec); if None then no time limit
         
         Raise RuntimeError if:
         - already connecting or connected
@@ -202,6 +206,7 @@ class TCPConnection(object):
             host = self.host,
             port = self.port,
             stateCallback = self._sockStateCallback,
+            timeLim = timeLim,
             name = self._name,
         )
         
@@ -244,6 +249,12 @@ class TCPConnection(object):
         """Return True if connected, False otherwise.
         """
         return self._state in self._ConnectedStates
+    
+    @property
+    def isDisconnected(self):
+        """Return True if fully disconnected, False otherwise.
+        """
+        return self._state in self._DisconnectedStates
 
     @property
     def isDone(self):
@@ -251,6 +262,12 @@ class TCPConnection(object):
         """
         return self._state in self._DoneStates
     
+    @property
+    def didFail(self):
+        """Return True if the connection failed
+        """
+        return self._state in self._FailedStates
+    
     @property
     def mayConnect(self):
         """Return True if one may call connect, false otherwise"""
