@@ -7,6 +7,10 @@ History:
 2012-07-19 ROwen
 2012-11-29 ROwen    Misfeature fix: write and writeLine failed on unicode strings that could be converted to ASCII
                     (a "feature" of Twisted Framework).
+2013-12-24 ROwen    Added timeLim argument to socket constructors.
+                    TCPSocket.port now returns the auto-selected port if constructed with port=0.
+                    Added experimental support for Deferreds; it's messy so I plan to remove it again.
+2014-01-01 ROwen    Removed experimental support for Deferreds.
 """
 __all__ = ["Socket", "TCPSocket", "Server", "TCPServer"]
 
@@ -67,7 +71,7 @@ class _SocketProtocol(Protocol):
         #print "%s.read(nChar=%r) returning %r; remaining buffer=%r" % (self, nChar, data, self.__buffer)
 
         if self.__buffer and self._readCallback:
-            Timer(0.000001, self._readCallback, self)
+            Timer(0, self._readCallback, self)
         return data
         
     def readLine(self, default=None):
@@ -82,7 +86,7 @@ class _SocketProtocol(Protocol):
         #print "%s.readLine(default=%r) returning %r; remaining buffer=%r" % (self, default, res[0], self.__buffer)
 
         if self.__buffer and self._readCallback:
-            Timer(0.000001, self._readCallback, self)
+            Timer(0, self._readCallback, self)
         return res[0]
         
     def dataReceived(self, data):
@@ -130,15 +134,6 @@ class _SocketProtocolFactory(Factory):
 
 class Socket(BaseSocket):
     """A socket using Twisted framework.
-    
-    Inputs:
-    - endpoint  a Twisted endpoint, e.g. twisted.internet.endpoints.TCP4ClientEndpoint;
-    - protocol: a Twisted protocol;
-        you must either specify endpoint or protocol, but not both
-    - state     the initial state
-    - readCallback: function to call when data read; receives: self
-    - stateCallback a state callback function; see addStateCallback for details
-    - name: a string to identify this socket; strictly optional
     """
     def __init__(self,
         endpoint = None,
@@ -146,8 +141,21 @@ class Socket(BaseSocket):
         state = BaseSocket.Connected,
         readCallback = nullCallback,
         stateCallback = nullCallback,
+        timeLim = None,
         name = "",
     ):
+        """Construct a Socket
+
+        Inputs:
+        - endpoint  a Twisted endpoint, e.g. twisted.internet.endpoints.TCP4ClientEndpoint;
+        - protocol  a Twisted protocol;
+            you must either specify endpoint or protocol, but not both
+        - state     the initial state
+        - readCallback  function to call when data read; receives: self
+        - stateCallback a state callback function; see addStateCallback for details
+        - timeLim   time limit to make connection (sec); no limit if None or 0
+        - name      a string to identify this socket; strictly optional
+        """
         #print "Socket(name=%r, endpoint=%r, protocol=%r, state=%r, readCallback=%r, stateCallback=%r)" % \
         #    (name, endpoint, protocol, state, readCallback, stateCallback)
         if bool(endpoint is None) == bool(protocol is None):
@@ -156,6 +164,7 @@ class Socket(BaseSocket):
         self._endpointDeferred = None
         self._protocol = None
         self._data = None
+        self._connectTimer = Timer()
         BaseSocket.__init__(self,
             state = state,
             readCallback = readCallback,
@@ -165,6 +174,8 @@ class Socket(BaseSocket):
         if protocol is not None:
             self._connectionMade(protocol)
         else:
+            if timeLim:
+                self._connectTimer.start(timeLim, self._connectTimeout)
             self._setState(BaseSocket.Connecting)
             self._endpointDeferred = self._endpoint.connect(_SocketProtocolFactory())
             setCallbacks(self._endpointDeferred, self._connectionMade, self._connectionLost)
@@ -195,23 +206,25 @@ class Socket(BaseSocket):
             self._endpointDeferred = None
         if self._protocol is not None:
             self._protocol.roAbort()
+            self._protocol = None
 
     def _connectionLost(self, reason):
         """Connection lost callback
         """
         #print "%s._connectionLost(reason=%r)" % (self, reason)
         if reason is None:
-            reasonStr = None
-        elif issubclass(getattr(reason, "type", None), ConnectionDone):
+            reasonStrOrNone = None
+        elif isinstance(reason, ConnectionDone):
             # connection closed cleanly; no need for a reason
             # use getattr in case reason as no type attribute
-            reasonStr = None
+            reasonStrOrNone = None
         else:
-            reasonStr = str(reason)
+            reasonStrOrNone = str(reason)
+
         if self._state == BaseSocket.Closing:
-            self._setState(BaseSocket.Closed, reasonStr)
+            self._setState(BaseSocket.Closed, reasonStrOrNone)
         else:
-            self._setState(BaseSocket.Failed, reasonStr)
+            self._setState(BaseSocket.Failed, reasonStrOrNone)
     
     def _connectionMade(self, protocol):
         """Callback when connection made
@@ -289,9 +302,17 @@ class Socket(BaseSocket):
             raise RuntimeError("%s not connected" % (self,))
         self.write(data + "\r\n")
     
+    def _connectTimeout(self):
+        """Call if connection times out
+        """
+        if not self.isReady or self.isDone:
+            self.close(isOK=False, reason="timeout")
+    
     def _doRead(self, sock):
         """Called when there is data to read
         """
+        if not self.isReady:
+            return
         if self._readCallback:
             try:
                 self._readCallback(self)
@@ -299,6 +320,11 @@ class Socket(BaseSocket):
                 sys.stderr.write("%s read callback %s failed: %s\n" % (self, self._readCallback, e,))
                 traceback.print_exc(file=sys.stderr)
 
+    def _setState(self, *args, **kwargs):
+        BaseSocket._setState(self, *args, **kwargs)
+        if self.isReady or self.isDone:
+            self._connectTimer.cancel()
+
 
 class TCPSocket(Socket):
     """A TCP/IP socket using Twisted framework.
@@ -308,6 +334,7 @@ class TCPSocket(Socket):
         port = None,
         readCallback = None,
         stateCallback = None,
+        timeLim = None,
         name = "",
     ):
         """Construct a TCPSocket
@@ -317,6 +344,7 @@ class TCPSocket(Socket):
         - port      the port
         - readCallback  function to call when data read; receives: self
         - stateCallback a state callback function; see addStateCallback for details
+        - timeLim   time limit to make connection (sec); no limit if None or 0
         - name      a string to identify this socket; strictly optional
         """
         endpoint = TCP4ClientEndpoint(reactor, host=host, port=port)
@@ -324,6 +352,7 @@ class TCPSocket(Socket):
             endpoint = endpoint,
             readCallback = readCallback,
             stateCallback = stateCallback,
+            timeLim = timeLim,
             name = name,
         )
 
@@ -336,10 +365,10 @@ class Server(BaseServer):
     """
     def __init__(self,
         endpoint,
-        connCallback = nullCallback,
-        stateCallback = nullCallback,
-        sockReadCallback = nullCallback,
-        sockStateCallback = nullCallback,
+        connCallback = None,
+        stateCallback = None,
+        sockReadCallback = None,
+        sockStateCallback = None,
         name = "",
     ):
         """Construct a socket server
@@ -365,16 +394,25 @@ class Server(BaseServer):
             sockStateCallback = sockStateCallback,
             name = name,
         )
+        self._numConn = 0
+        Timer(0, self._startListening) # let device be fully constructed before listening
+            # I'm not sure why this is needed, but it seems to help
+    
+    def _startListening(self):
+        """Start listening for new connections
+        """
         self._endpointDeferred = self._endpoint.listen(_SocketProtocolFactory(self._newConnection))
         setCallbacks(self._endpointDeferred, self._listeningCallback, self._connectionLost)
-        self._numConn = 0
 
     @property
     def port(self):
         """Return the port, or None if not known
         """
-        return getattr(self._endpoint, "_port", None)
-    
+        port = getattr(self._protocol, "port", None)
+        if port == 0: # try an undocumented interface
+            port = getattr(self._protocol, "_realPortNumber", None)
+        return port
+
     def _listeningCallback(self, protocol):
         self._protocol = protocol
         self._setState(self.Listening)
@@ -402,18 +440,22 @@ class Server(BaseServer):
         """Connection failed callback
         """
         #print "%s._connectionLost(%s)" % (self, reason)
+        if self.isDone:
+            return
+
         if reason is None:
-            reasonStr = None
+            reasonStrOrNone = None
         elif issubclass(getattr(reason, "type", None), ConnectionDone):
             # connection closed cleanly; no need for a reason
             # use getattr in case reason as no type attribute
-            reasonStr = None
+            reasonStrOrNone = None
         else:
-            reasonStr = str(reason)
+            reasonStrOrNone = str(reason)
+
         if self._state == BaseSocket.Closing:
-            self._setState(BaseSocket.Closed, reasonStr)
+            self._setState(BaseSocket.Closed, reasonStrOrNone)
         else:
-            self._setState(BaseSocket.Failed, reasonStr)
+            self._setState(BaseSocket.Failed, reasonStrOrNone)
     
     def _newConnection(self, protocol):
         """A client has connected. Create a Socket and call the connection callback with it.
@@ -440,16 +482,16 @@ class TCPServer(Server):
     """
     def __init__(self,
         port,
-        connCallback = nullCallback,
-        stateCallback = nullCallback,
-        sockReadCallback = nullCallback,
-        sockStateCallback = nullCallback,
+        connCallback = None,
+        stateCallback = None,
+        sockReadCallback = None,
+        sockStateCallback = None,
         name = "",
     ):
         """Construct a socket server
         
         Inputs:
-        - endpoint: a Twisted endpoint, e.g. twisted.internet.endpoints.TCP4ClientEndpoint
+        - port: the port on which to listen for new connections
         - connCallback: function to call when a client connects; it receives the following arguments:
                     - sock, a Socket
         - stateCallback: function to call when server changes state; it receives one argument: this server
