@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-"""A widget to changing values in real time as a strip chart
+"""A widget to display changing values in real time as a strip chart
 
 Known issues:
 Matplotlib's defaults present a number of challenges for making a nice strip chart display.
@@ -47,26 +47,29 @@ Here are manual workarounds for some common problems:
 
 Requirements:
 - Requires matplotlib built with TkAgg support
-- This widget is somewhat experimental and the API may change.
 
 Acknowledgements:
 I am grateful to Benjamin Root, Tony S Yu and others on matplotlib-users
 for advice on tying the x axes together and improving the layout.
 
 History:
-2010-09-29  ROwen
-2010-11-30  Fixed a memory leak (Line._purgeOldData wasn't working correctly).
-2010-12-10  Document a memory leak caused by matplotlib's canvas.draw.
-2010-12-23  Backward-incompatible changes:
-            - addPoint is now called on the object returned by addLine, not StripChartWdg.
-                This eliminate the need to give lines unique names.
-            - addPoint is silently ignored if y is None
-            - addLine and addConstantLine have changed:
-                - There is no "name" argument; use label if you want a name that shows up in legends.
-                - The label does not have to be unique.
-                - They return an object.
-            Added removeLine method.
-2010-12-29  Document useful arguments for addLine.
+2010-09-29 ROwen
+2010-11-30 ROwen    Fixed a memory leak (Line._purgeOldData wasn't working correctly).
+2010-12-10 ROwen    Document a memory leak caused by matplotlib's canvas.draw.
+2010-12-23 ROwen    Backward-incompatible changes:
+                    - addPoint is now called on the object returned by addLine, not StripChartWdg.
+                        This eliminate the need to give lines unique names.
+                    - addPoint is silently ignored if y is None
+                    - addLine and addConstantLine have changed:
+                        - There is no "name" argument; use label if you want a name that shows up in legends.
+                        - The label does not have to be unique.
+                        - They return an object.
+                    Added removeLine method.
+2010-12-29 ROwen    Document useful arguments for addLine.
+2012-05-31 ROwen    Add a clear method to StripChartWdg and _Line.
+2012-06-04 ROwen    Reduce CPU usage by doing less work if not visible (not mapped).
+2012-07-09 ROwen    Modified to use RO.TkUtil.Timer.
+2012-09-18 ROwen    Explicitly import matplotlib.dates to avoid a problem with matplotlib 1.2.0rc1
 """
 import bisect
 import datetime
@@ -75,7 +78,9 @@ import time
 import numpy
 import Tkinter
 import matplotlib
+import matplotlib.dates
 from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
+from RO.TkUtil import Timer
 
 __all__ = ["StripChartWdg"]
 
@@ -136,6 +141,8 @@ class StripChartWdg(Tkinter.Frame):
         Tkinter.Frame.__init__(self, master)
         
         self._timeRange = timeRange
+        self._isVisible = self.winfo_ismapped()
+        self._isFirst = True
         if updateInterval == None:
             updateInterval = max(0.1, min(5.0, timeRange / 2000.0))
         self.updateInterval = float(updateInterval)
@@ -178,7 +185,9 @@ class StripChartWdg(Tkinter.Frame):
             subplot.label_outer() # disable axis labels on all but the bottom subplot
             subplot.set_ylim(auto=True) # set auto scaling for the y axis
         
-        self._timeAxisTimer = None
+        self.bind("<Map>", self._handleMap)
+        self.bind("<Unmap>", self._handleUnmap)
+        self._timeAxisTimer = Timer()
         self._updateTimeAxis()
 
     def addConstantLine(self, y, subplotInd=0, **kargs):
@@ -213,7 +222,18 @@ class StripChartWdg(Tkinter.Frame):
           Arguments to avoid include: animated, data, xdata, ydata, zdata, figure.
         """
         subplot = self.subplotArr[subplotInd]
-        return _Line(subplot, self._cnvTimeFunc, **kargs)
+        return _Line(
+            subplot = subplot,
+            cnvTimeFunc = self._cnvTimeFunc,
+            wdg = self,
+        **kargs)
+    
+    def clear(self):
+        """Clear data in all non-constant lines
+        """
+        for subplot in self.subplotArr:
+            for line in subplot._scwLines:
+                line.clear()
 
     def getDoAutoscale(self, subplotInd=0):
         return self.subplotArr[subplotInd].get_autoscaley_on()
@@ -285,7 +305,7 @@ class StripChartWdg(Tkinter.Frame):
             subplot.relim()
             subplot.autoscale_view(scalex=False, scaley=True)
 
-    def _handleDrawEvent(self, event):
+    def _handleDrawEvent(self, event=None):
         """Handle draw event
         """
 #         print "handleDrawEvent"
@@ -295,12 +315,21 @@ class StripChartWdg(Tkinter.Frame):
                 subplot.draw_artist(line.line2d)
             self.canvas.blit(subplot.bbox)
     
+    def _handleMap(self, evt):
+        """Handle map event (widget made visible)
+        """
+        self._isVisible = True
+        self._handleDrawEvent()
+        self._updateTimeAxis()
+    
+    def _handleUnmap(self, evt):
+        """Handle unmap event (widget made not visible)
+        """
+        self._isVisible = False
+    
     def _updateTimeAxis(self):
         """Update the time axis; calls itself
         """
-        if self._timeAxisTimer != None:
-            self.after_cancel(self._timeAxisTimer)
-            self._timeAxisTimer = None
         tMax = time.time() + self.updateInterval
         tMin = tMax - self._timeRange
         minMplDays = self._cnvTimeFunc(tMin)
@@ -308,18 +337,23 @@ class StripChartWdg(Tkinter.Frame):
         
         self._purgeCounter = (self._purgeCounter + 1) % self._maxPurgeCounter
         doPurge = self._purgeCounter == 0
-        
-        for subplot in self.subplotArr:
-            subplot.set_xlim(minMplDays, maxMplDays)
-            if doPurge:
+
+        if doPurge:
+            for subplot in self.subplotArr:
                 for line in subplot._scwLines:
                     line._purgeOldData(minMplDays)
-                if subplot.get_autoscaley_on():
-                    # since data is being purged the y limits may have changed
-                    subplot.relim()
-                    subplot.autoscale_view(scalex=False, scaley=True)
-        self.canvas.draw()
-        self._timeAxisTimer = self.after(int(self.updateInterval * 1000), self._updateTimeAxis)
+        
+        if self._isVisible or self._isFirst:
+            for subplot in self.subplotArr:
+                subplot.set_xlim(minMplDays, maxMplDays)
+                if doPurge:
+                    if subplot.get_autoscaley_on():
+                        # since data is being purged the y limits may have changed
+                        subplot.relim()
+                        subplot.autoscale_view(scalex=False, scaley=True)
+            self._isFirst = False
+            self.canvas.draw()
+        self._timeAxisTimer.start(self.updateInterval, self._updateTimeAxis)
 
 
 class _Line(object):
@@ -331,17 +365,19 @@ class _Line(object):
     - cnvTimeFunc: a function that takes a POSIX timestamp (e.g. time.time()) and returns matplotlib days;
         typically an instance of TimeConverter; defaults to TimeConverter(useUTC=False)
     """
-    def __init__(self, subplot, cnvTimeFunc, **kargs):
+    def __init__(self, subplot, cnvTimeFunc, wdg, **kargs):
         """Create a line
         
         Inputs:
         - subplot: the matplotlib Subplot instance displaying this line
         - cnvTimeFunc: a function that takes a POSIX timestamp (e.g. time.time()) and returns matplotlib days;
             typically an instance of TimeConverter; defaults to TimeConverter(useUTC=False)
+        - wdg: parent strip chart widget; used to test visibility
         - **kargs: keyword arguments for matplotlib Line2D, such as color
         """
         self.subplot = subplot
         self._cnvTimeFunc = cnvTimeFunc
+        self._wdg = wdg
         # do not use the data in the Line2D because in some versions of matplotlib
         # line.get_data returns numpy arrays, which cannot be appended to
         self._tList = []
@@ -365,15 +401,24 @@ class _Line(object):
 
         self._tList.append(mplDays)
         self._yList.append(y)
-        if self.subplot.get_autoscaley_on() and numpy.isfinite(y):
-            yMin, yMax = self.subplot.get_ylim()
-            self.line2d.set_data(self._tList, self._yList)
-            if not (yMin <= y <= yMax):
-                self.subplot.relim()
-                self.subplot.autoscale_view(scalex=False, scaley=True)
-                return # a draw event was triggered
-        else:
-            self.line2d.set_data(self._tList, self._yList)
+        self._redraw()
+    
+    def _redraw(self):
+        """Redraw the graph
+        """
+        self.line2d.set_data(self._tList, self._yList)
+        if not self._wdg.winfo_ismapped():
+            return
+        if len(self._yList) > 0:
+            # see if limits need updating to include last point
+            lastY = self._yList[-1]
+            if self.subplot.get_autoscaley_on() and numpy.isfinite(lastY):
+                yMin, yMax = self.subplot.get_ylim()
+                self.line2d.set_data(self._tList, self._yList)
+                if not (yMin <= lastY <= yMax):
+                    self.subplot.relim()
+                    self.subplot.autoscale_view(scalex=False, scaley=True)
+                    return # a draw event was triggered
 
         # did not trigger redraw event so do it now
         if self.subplot._scwBackground:
@@ -383,6 +428,13 @@ class _Line(object):
                 self.subplot.draw_artist(line.line2d)
             canvas.blit(self.subplot.bbox)
 
+    def clear(self):
+        """Clear all data
+        """
+        self._tList = []
+        self._yList = []
+        self._redraw()
+
     def _purgeOldData(self, minMplDays):
         """Purge data with t < minMplDays
 
@@ -459,14 +511,19 @@ if __name__ == "__main__":
         walk1Line:  RO.Alg.RandomWalk.GaussianRandomWalk(0, 2),
         walk2Line: RO.Alg.RandomWalk.GaussianRandomWalk(0, 2),
     }
-    def addRandomValues(line, interval=100):
+    def addRandomValues(line, interval=0.1):
+        """Add random values to the specified strip chart line
+        Inputs:
+        - line: strip chart line
+        - interval: interval between updates (sec)
+        """
         var = varDict[line]
         line.addPoint(var.next())
-        root.after(interval, addRandomValues, line, interval)
+        Timer(interval, addRandomValues, line, interval)
 
-    addRandomValues(countsLine, interval=500)
-    addRandomValues(walk1Line, 1600)
-    addRandomValues(walk2Line, 1900)
+    addRandomValues(countsLine, interval=0.5)
+    addRandomValues(walk1Line, 1.6)
+    addRandomValues(walk2Line, 1.9)
     
     def deleteSatConstLine():
         stripChart.removeLine(satConstLine)
